package com.jiayue.ipfcst.common.data.entity;

import lombok.Data;
import org.hibernate.annotations.GenericGenerator;
import org.springframework.core.annotation.Order;
import org.springframework.data.annotation.CreatedDate;

import javax.persistence.*;
import javax.validation.constraints.Digits;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 数值天气预报历史数据
 *
 * @author zzy
 * @version 1.0
 * @since 2019/11/28 16:05
 */
@Data
@Entity
public class NwpHis {
    @Id
    @Order(1)
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "myid")
    @GenericGenerator(name = "myid", strategy = "com.jiayue.ipfcst.common.data.entity.id.CustomIDGenerator")
    private Integer id;

    /*提前多久预测*/
    @Column
    private Integer forecastHowLongAgo;
    /**
     * 历史数据产生时间
     */
    @Order(100)
    @Temporal(TemporalType.TIMESTAMP)
    @CreatedDate
    @Column
    private Date genDate;

    /**
     * 预测时间
     */
    @Column
    private Long preTime;

    /**
     * 场站ID
     */
    @Column
    private String farmId;
    /**
     * 生成日期
     */
    @Column
    private String scDate;
    /**
     * 生成时间
     */
    @Column
    private String scTime;
    /**
     * 预测日期
     */
    @Column
    private String preDate;

    /**
     * 温度
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal t = new BigDecimal(-99);
    /**
     * 湿度
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal rh = new BigDecimal(-99);
    /**
     * 气压
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal pressure = new BigDecimal(-99);
    /**
     * 辐射
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal swr = new BigDecimal(-99);

    /**
     * 短波辐射
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal lwr = new BigDecimal(-99);
    /**
     * 直辐射
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal directRadiation = new BigDecimal(-99);
    /**
     * 散辐射
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal diffuseRadiation = new BigDecimal(-99);
    /**
     * 热感通量
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal senf = new BigDecimal(-99);
    /**
     * 10米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws10 = new BigDecimal(-99);
    /**
     * 30米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws30 = new BigDecimal(-99);
    /**
     * 50米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws50 = new BigDecimal(-99);
    /**
     * 70米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws70 = new BigDecimal(-99);
    /**
     * 80米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws80 = new BigDecimal(-99);
    /**
     * 90米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws90 = new BigDecimal(-99);
    /**
     * 100米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws100 = new BigDecimal(-99);
    /**
     * 170米风速
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws170 = new BigDecimal(-99);
    /**
     * 10米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd10 = new BigDecimal(-99);
    /**
     * 30米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd30 = new BigDecimal(-99);
    /**
     * 50米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd50 = new BigDecimal(-99);
    /**
     * 70米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd70 = new BigDecimal(-99);
    /**
     * 80米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd80 = new BigDecimal(-99);
    /**
     * 90米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd90 = new BigDecimal(-99);
    /**
     * 100米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd100 = new BigDecimal(-99);
    /**
     * 170米风向
     */
    @Column
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd170 = new BigDecimal(-99);
}
