/** When your routing table is too long, you can split it into small modules**/

import Layout from '@/layout'

const consoleRouter = {
  path: '/console',
  component: Layout,
  redirect: '/console/electricFieldInfo',
  name: 'console',
  meta: {
    title: '控制管理',
    icon: 'table'
  },
  children: [
    {
      path: 'electricField',
      component: () => import('@/views/console/electricField/index'),
      name: 'electricField',
      meta: { title: '场站信息', noCache: true },
      sign: 'currency'
    },
    {
      path: 'pvModuleModel',
      component: () => import('@/views/console/pvModuleModel/index'),
      name: 'pvModuleModel',
      meta: { title: '光伏组件', noCache: true },
      sign: 'photovoltaic'
    },
    {
      path: 'inverterInfo',
      component: () => import('@/views/console/inverterInfo/index'),
      name: 'inverterInfo',
      meta: { title: '逆变器信息', noCache: true },
      sign: 'photovoltaic'
    },
    {
      path: 'holidayCalendar',
      component: () => import('@/views/console/holidayCalendar/index'),
      name: 'holidayCalendar',
      meta: { title: '假期日历', noCache: true },
      sign: 'currency'
    },
    {
      path: 'weatherStationInfo',
      component: () => import('@/views/console/weatherStationInfo/index'),
      name: 'WeatherStationInfo',
      meta: { title: '气象站信息', noCache: true },
      sign: 'photovoltaic'
    },
    {
      path: 'anemometerTowerInfo',
      component: () => import('@/views/console/windTowerInfo/index'),
      name: 'AnemometerTowerInfo',
      meta: { title: '测风塔信息', noCache: true },
      sign: 'windPower'
    },
    {
      path: 'agc_avcInfo',
      component: () => import('@/views/console/agc_avcInfo/index'),
      name: 'agc_avcInfo',
      meta: { title: 'AGC_AVC信息', noCache: true },
      sign: 'currency'
    },
    {
      path: 'subsidiaryEquipmentInfo',
      component: () => import('@/views/console/subsidiaryEquipmentInfo/index'),
      name: 'subsidiaryEquipmentInfo',
      meta: { title: '附属设备信息', noCache: true },
      sign: 'currency'
    },
    {
      path: 'boosterStationInfo',
      component: () => import('@/views/console/boosterStationInfo/index'),
      name: 'boosterStationInfo',
      meta: { title: '升压站信息', noCache: true },
      sign: 'currency'
    },
    {
      path: 'windTurbineidInfo',
      component: () => import('@/views/console/windTurbineInfo/index'),
      name: 'WindTurbineInfo',
      meta: { title: '风机信息', noCache: true },
      sign: 'windPower'
    },
    {
      path: 'overHaulPlan',
      component: () => import('@/views/console/overHaulPlan/index'),
      name: 'OverHaulPlan',
      meta: { title: '光伏检修计划', noCache: true },
      sign: 'currency'
    },
    {
      path: 'overHaulPlanOne',
      component: () => import('@/views/console/overHaulPlanOne/index'),
      name: 'OverHaulPlanOne',
      meta: { title: '风电检修计划', noCache: true },
      sign: 'currency'
    },
    // {
    //   path: 'powerPlanSD',
    //   component: () => import('@/views/console/powerPlanSD/index'),
    //   name: 'powerPlanSD',
    //   meta: { title: '山东发电计划', noCache: true },
    //   sign: 'currency'
    // },
    // {
    //   path: 'consoleCenter',
    //     component: () => import('@/views/console/consoleCenter/index'),
    //   name: 'ConsoleCenter',
    //   meta: {title: '数据控制', noCache: true}
    // },
    {
      path: 'sysParameter',
      component: () => import('@/views/console/sysParameter/index'),
      name: 'SysParameter',
      meta: { title: '系统参数', noCache: true },
      sign: 'currency'
    },
    {
      path: 'sysAlarm',
      component: () => import('@/views/console/sysAlarm/index'),
      name: 'SysAlarm',
      meta: { title: '系统告警', noCache: true },
      sign: 'currency'
    },
    // {
    //   path: 'manualInterventionData',
    //   component: () => import('@/views/console/manualInterventionData/index'),
    //   name: 'ManualInterventionData',
    //   meta: { title: '人工干预设置', noCache: true },
    //   sign: 'currency'
    // },
    // {
    //   path: 'manualInterventionQuery',
    //   component: () => import('@/views/console/manualInterventionQuery/index'),
    //   name: 'ManualInterventionQuery',
    //   meta: { title: '人工干预数据查询', noCache: true },
    //   sign: 'currency'
    // },
    // {
    //   path: 'dataLimitation',
    //   component: () => import('@/views/console/dataLimitation/index'),
    //   name: 'DataLimitation',
    //   meta: { title: '上报数据限制', noCache: true },
    //   sign: 'currency'
    // },
    // {
    //   path: 'dataLimitationLog',
    //   component: () => import('@/views/console/dataLimitationLog/index'),
    //   name: 'DataLimitationLog',
    //   meta: { title: '上报数据操作记录', noCache: true },
    //   sign: 'currency'
    // },
    // {
    //   path: 'communicationEquipment',
    //   component: () => import('@/views/console/communicationEquipment/index'),
    //   name: 'CommunicationEquipment',
    //   meta: { title: '通信设备', noCache: true },
    //   sign: 'currency'
    // },
    {
      path: 'fileAnalysisRecord',
      component: () => import('@/views/console/FileAnalysisRecord/index'),
      name: 'FileAnalysisRecord',
      meta: { title: '文件解析记录', noCache: true },
      sign: 'currency'
    },
    // {
    //   path: 'QREnvironmentData',
    //   component: () => import('@/views/console/QREnvironmentData/index'),
    //   name: 'QREnvironmentData',
    //   meta: { title: '二维码数据回传', noCache: true },
    //   sign: 'currency'
    // }
    // {
    //   path: 'uploadFileChannel',
    //     component: () => import('@/views/console/UploadFileChannel/index'),
    //   name: 'UploadFileChannel',
    //   meta: {title: '通道配置', noCache: true}
    // }

  ]
}

export default consoleRouter
