package com.jiayue.ipfcst.console.util;

import com.jiayue.ipfcst.common.core.util.DateMomentUtil;
import com.jiayue.ipfcst.common.data.constant.enums.ElectricFieldTypeEnum;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * 常量类
 *
 * @author bizy
 * @version 3.0
 */
public class Constant {

  /*参数集合*/
  public static Map<String, String> sysParameterList = new Hashtable<String, String>();

  public static final String suffix = ".csv";

  /*风预测系数*/
  public static List<String> windCoeList = Arrays.asList("0.3358", "0.335", "0.3352", "0.3366", "0.339", "0.342", "0.3456", "0.35", "0.3554", "0.3614", "0.3664", "0.3706", "0.3738", "0.3766", "0.3792", "0.3814", "0.3836", "0.3852", "0.3868", "0.388", "0.3892", "0.3894", "0.389", "0.3878", "0.3854", "0.381", "0.375", "0.3674", "0.358", "0.347", "0.334", "0.3186", "0.301", "0.2826", "0.2646", "0.2468", "0.229", "0.212", "0.1954", "0.1794", "0.1638", "0.1488", "0.1344", "0.1204", "0.107", "0.094", "0.0816", "0.0702", "0.0596", "0.0498", "0.041", "0.033", "0.0256", "0.0194", "0.0142", "0.0104", "0.0078", "0.0058", "0.0042", "0.0032", "0.0022", "0.0016", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0016", "0.0016", "0.0016", "0.0016", "0.0016", "0.0016", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0014", "0.0018", "0.0024", "0.0032", "0.0046", "0.0062", "0.0082", "0.011", "0.0144", "0.0188", "0.024", "0.0296", "0.0356");
  /*光预测系数*/
  public static List<String> photovoltaicCoeList = Arrays.asList("0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0.0097", "0.0272", "0.0504", "0.0778", "0.1103", "0.1572", "0.2032", "0.2540", "0.3088", "0.3781", "0.4350", "0.4943", "0.5422", "0.5901", "0.6314", "0.6636", "0.6901", "0.6980", "0.7119", "0.7206", "0.7244", "0.7186", "0.7121", "0.7109", "0.6996", "0.6930", "0.6740", "0.6569", "0.6361", "0.6365", "0.6058", "0.5794", "0.5434", "0.5240", "0.4824", "0.4403", "0.3986", "0.3764", "0.3358", "0.2890", "0.2464", "0.2207", "0.1802", "0.1439", "0.1138", "0.0985", "0.0748", "0.0545", "0.0358", "0.0238", "0.0102", "0.0019", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0", "0");


  /**
   * 功能描述: 获取系统参数方法
   *
   * @param key          参数key
   * @param defaultValue 默认值
   * @return 参数value
   */
  public static String getValue(String key, String defaultValue) {
    if (key != null && !key.isEmpty()) {
      if (sysParameterList.containsKey(key)) {
        defaultValue = sysParameterList.get(key);
      }
    }
    return defaultValue;
  }

  /**
   * 获取预测系数
   *
   * @param forecastTime          预测时间
   * @param electricFieldTypeEnum 场站类型
   * @return
   */
  public static String forecastCoe(Long forecastTime, ElectricFieldTypeEnum electricFieldTypeEnum) {
    int point = DateMomentUtil.getMoment(forecastTime, 15 * 60 * 1000L);
    String coe = "";
    if (ElectricFieldTypeEnum.E1.equals(electricFieldTypeEnum)) {
      coe = photovoltaicCoeList.get(point - 1);
    } else {
      coe = windCoeList.get(point - 1);
    }
    return coe;
  }
}
